/*------------------------------------------------------------------------
    File        : CustomerEventHandler.p
    Purpose     : Presentation Layer Event Handler for Customer functionality 
    Author(s)   : pjudge
    Created     : Tue May 10 16:13:56 EDT 2011
    Notes       :
  ----------------------------------------------------------------------*/
routine-level on error undo, throw.

using Progress.Lang.Object.

class CustomerEventHandler implements IEventHandler:
    
    define private temp-table ttComponent no-undo
        field Type as character /* CLIENTDATA, UI */
        field Name as character /* logical name */
        field ClassName as character /* .CLS name */
        field Reference as Object
        index idx1 as primary unique Type Name
        . 
    
    define public property UIName as character no-undo get. set.

    method protected IUserInterface GetCurrentUI():
        find ttComponent where
             ttComponent.Type eq 'UI' and
             ttComponent.Name eq UIName 
             no-lock no-error.
        if available ttComponent then
            return cast(ttComponent.Reference, IUserInterface).
    end method.

    constructor public CustomerEventHandler():    
        RegisterComponents().
    end constructor.
    
    /*------------------------------------------------------------------------------
      Purpose:     Figure out what the initial set of data is for this event handler.
      Parameters:  <none>
      Notes:       
    ------------------------------------------------------------------------------*/
    method protected void GetInitialData():
        define variable cFieldName as character extent 3 no-undo.
        define variable cFieldValue as character extent 3 no-undo.
        
        /* these fields will only apprea in cerain cases (ie Web) so
           typically, we'll just GetFirst(customer). */
        assign cFieldName[1] = 'CurrentCustNum'
               cFieldName[2] = 'NavPanel'
               cFieldName[3] = 'UpdatePanel'.
        
        cFieldValue = GetCurrentUI():GetScreenValue(input cFieldName).
        case cFieldValue[2]:
            when 'next' then
            do:
                if cFieldValue[1] ne '' then 
                    RepositionQuery('customer', 'CustNum', cFieldValue[1]).
                 GetNext('customer').
            end.
            when 'prev' then
            do:
                if cFieldValue[1] ne '' then 
                    RepositionQuery('customer', 'CustNum', cFieldValue[1]).
                GetPrev('customer').
            end.
            when 'last' then
                GetLast('customer').
            otherwise
                GetFirst('customer').
        end case.
        
    end method.

    method protected void RegisterComponents():
        /* registered components */
        create ttComponent.
        assign ttComponent.Type = 'CLIENTDATA'
               ttComponent.Name = 'customer'
               ttComponent.ClassName = 'CustomerClientData'
               .
        
        create ttComponent.
        assign ttComponent.Type = 'UI'
               ttComponent.Name = 'ABLGUI'
               ttComponent.ClassName = 'CustomerAblGuiWrapper'
               .
        create ttComponent.
        assign ttComponent.Type = 'UI'
               ttComponent.Name = 'DOTNET'
               ttComponent.ClassName = 'CustomerFormView'
               .
        create ttComponent.
        assign ttComponent.Type = 'UI'
               ttComponent.Name = 'WEBSPEED'
               ttComponent.ClassName = 'CustomerWebUI'
               .
    end method.
    
    method protected void UpdateButtonState (input poClientData as IClientData):
        /* [first][prev][next][last] */
        define variable lEnabled as logical extent 4 no-undo.
        define variable cRowPosition as character no-undo.
        
        cRowPosition = poClientData:GetCurrentPosition().
    
        /* Decide what the logical state of the buttons are based on CLIENTDATA info */        
        case cRowPosition:
            when 'IsFirstAndLast ' then
                assign lEnabled[1] = false
                       lEnabled[2] = false
                       lEnabled[3] = false
                       lEnabled[4] = false.
            when 'IsLast' then
                assign lEnabled[1] = true
                       lEnabled[2] = true
                       lEnabled[3] = false
                       lEnabled[4] = false.
           when 'IsFirst' then
            assign lEnabled[1] = false
                   lEnabled[2] = false
                   lEnabled[3] = true
                   lEnabled[4] = true.
            when 'NotFirstOrLast' then               
            assign lEnabled[1] = true
                   lEnabled[2] = true
                   lEnabled[3] = true
                   lEnabled[4] = true.
            otherwise
                assign lEnabled[1] = false
                       lEnabled[2] = false
                       lEnabled[3] = false
                       lEnabled[4] = false.
        end case.
            
        /* tell the UI what to do */
        GetCurrentUI():SetNavPanelState(lEnabled).
    end method.
    
    method public void BeginWaitFor (input pcShowMode as character):
        GetCurrentUI():BeginWaitFor(pcShowMode).
    end method.
    
    method public void EndWaitFor():
        GetCurrentUI():EndWaitFor().
        Destroy().
    end method.
    
    method public void ShowUI():
        GetCurrentUI():ShowUI().
    end method.

    method public void HideUI():
        GetCurrentUI():ShowUI().
    end method.
    
    method public void Destroy():
        find ttComponent where 
             ttComponent.Type eq 'UI' and
             ttComponent.Name eq UIName
             no-error.
        if available ttComponent and valid-object(ttComponent.Reference) then
        do:
            GetCurrentUI():Destroy().
            ttComponent.Reference = ?.
        end.
        
        for each ttComponent where ttComponent.Type eq 'CLIENTDATA' and valid-object(ttComponent.Reference):
            cast(ttComponent.Reference, IClientData):Destroy().
            ttComponent.Reference = ?.
        end.
    end method.
    
    method public void Initialise():        
        /* start and initialise all the components */
        InitialiseComponents().
        
        GetInitialData().
        
        /* show the UI */
        ShowUI().
    end method.
    
    method protected void InitialiseComponents():
        /* start the Clientdata objects */            
        for each ttComponent where ttComponent.Type eq 'CLIENTDATA':
            ttComponent.Reference = dynamic-new(ttComponent.ClassName) ().
            
            cast(ttComponent.Reference, IClientData):Initialise().
        end.
        
        find ttComponent where
             ttComponent.Type eq 'UI' and
             ttComponent.Name eq UIName 
             no-lock no-error.
        if available ttComponent then
        do:
            ttComponent.Reference = dynamic-new(ttComponent.ClassName) ().
            cast(ttComponent.Reference, IUserInterface):EventHandler = this-object.
            cast(ttComponent.Reference, IUserInterface):Initialise().
        end.
    end method.
    
    method public void GetFirst(input pcClientDataName as character):
        find ttComponent where 
             ttComponent.Type eq 'CLIENTDATA' and
             ttComponent.Name eq pcClientDataName 
             no-lock no-error.
        if available ttComponent then
        do:
            /* reposition the model to the correct record */
            cast(ttComponent.Reference, IClientData):GetFirst().
            
            /* tell the view to get the data it needs */
            GetCurrentUI():ShowRecord().
            
            /* apply UI logic to the navigation panel buttons */
            UpdateButtonState(cast(ttComponent.Reference, IClientData)).
        end.
    end method.
    
    method public void GetPrev (input pcClientDataName as character):
        find ttComponent where 
             ttComponent.Type eq 'CLIENTDATA' and
             ttComponent.Name eq pcClientDataName 
             no-lock no-error.
        if available ttComponent then
        do:    
            cast(ttComponent.Reference, IClientData):GetPrev().
            GetCurrentUI():ShowRecord().
            UpdateButtonState(cast(ttComponent.Reference, IClientData)).
        end.                   
    end method.
    
    method public void GetNext (input pcClientDataName as character):
        find ttComponent where 
             ttComponent.Type eq 'CLIENTDATA' and
             ttComponent.Name eq pcClientDataName 
             no-lock no-error.
        if available ttComponent then
        do:    
            cast(ttComponent.Reference, IClientData):GetNext().
            GetCurrentUI():ShowRecord().
            UpdateButtonState(cast(ttComponent.Reference, IClientData)).
        end.                   
    end method.
    
    method public void GetLast (input pcClientDataName as character):
        find ttComponent where 
             ttComponent.Type eq 'CLIENTDATA' and
             ttComponent.Name eq pcClientDataName 
             no-lock no-error.
        if available ttComponent then
        do:
            cast(ttComponent.Reference, IClientData):GetLast().
            GetCurrentUI():ShowRecord(). 
            UpdateButtonState(cast(ttComponent.Reference, IClientData)).
        end.
    end method.
    
    method public character extent GetDataValue(input pcClientDataName as character,
                                                input pcFieldName as character extent):
        define variable cValue as character extent no-undo.
        
        find ttComponent where 
             ttComponent.Type eq 'CLIENTDATA' and
             ttComponent.Name eq pcClientDataName 
             no-lock no-error.
        if available ttComponent then
            cValue = cast(ttComponent.Reference, IClientData):GetDataValue(pcFieldName).
            
        return cValue.                    
    end method.
    
    method public void RepositionQuery(input pcClientDataName as character,
                                       input pcFieldName as character,
                                       input pcValue as character):
        find ttComponent where 
             ttComponent.Type eq 'CLIENTDATA' and
             ttComponent.Name eq pcClientDataName 
             no-lock no-error.
        if available ttComponent then
            cast(ttComponent.Reference, IClientData):RepositionQuery(pcFieldName, pcValue). 
    end method.
    
end class.

/* eof */